/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     TopicLoad.java                                             *
 *                                                                           *
 *  DESCRIPTION:  This application publishes a number of messages containing *
 *                a random integer user property called 'value' to a spec-   *
 *                ified Topic.                                               *
 *                                                                           *
 *                Further information about this class and its usage can be  *
 *                found in the 'Websphere MQ Using Java' documentation.      *
 *                                                                           *
 *  USAGE:        java TopicLoad -nojndi [-qm <qMgrName>] [-t <tName>]       *
 *             or                                                            *
 *                java TopicLoad [-icf <jndiICF>] [-url <jndiURL>]           *
 *                               [-tcflookup <tcfLookup>]                    *
 *                               [-tlookup <tLookup>]                        *
 *                                                                           *
 *                Both forms can additionally take the following arguments:  *
 *                               [-sleep <sleepTime>] [-msgs <numMsgs>]      *
 *                                                                           *
 *                Defaults for these parameters can be found below.          *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;
import javax.naming.*;
import java.util.*;

public class TopicLoad {
   // These are the defaults for the parameters which can be over-ridden
   // on the command line

   // Will we use JNDI for administered object creation?
   private static boolean useJNDI   = true;
   // JNDI parameters
   private static String  tcfLookup = "cn=tcf";
   private static String  tLookup   = "cn=t";
   private static String  icf       = "com.sun.jndi.ldap.LdapCtxFactory";
   private static String  url       = "ldap://localhost/o=ibm,c=us";
   // Non-JNDI parameters
   private static String  qMgrName  = "";
   private static String  tName     = "MQJMS/ASF/TopicLoad";
   // Other parameters
   private static int     numMsgs   = 200;
   private static int     sleepTime = 0;        // Default is to not sleep
                                                // between puts

   /**************************************************************************
    *  main - Application entry point                                        *
    **************************************************************************/
   public static void main( String[] args ) throws JMSException {
      // Parse the command line
      parseArguments(args);

      // Obtain the two administered objects we need
      Object[] adminObjects = new Object[2];
      obtainAdminObjects( adminObjects );
      TopicConnectionFactory tcf = (TopicConnectionFactory)adminObjects[0];
      Topic t = (Topic)adminObjects[1];

      // Create a TopicConnection and start it
      TopicConnection conn = tcf.createTopicConnection();
      conn.start();

      // Create a TopicSession
      TopicSession sess = conn.createTopicSession( false,
                                                   Session.AUTO_ACKNOWLEDGE );
      // Get a publisher
      TopicPublisher publisher = sess.createPublisher(t);

      // Display the destination we are using (requires access to MQJMS-
      // specific properties)
      System.out.println("\nTarget Broker Queue Manager : '"+
         ((com.ibm.mq.jms.MQTopicConnectionFactory)tcf).getBrokerQueueManager()
                                                            +"'");
      System.out.println("Target Topic                : '"+
                  ((com.ibm.mq.jms.MQTopic)t).getBaseTopicName()+"'");

      // Create a message to repeatedly send
      TextMessage msg = sess.createTextMessage();
      msg.setText("A message from the 'TopicLoad' sample application");

      System.out.print("Publishing messages         : ");

      // Obtain a random number generator
      Random rnd = new Random();

      // Publish the message 'numMsgs' times, with a sleep of 'sleep' ms
      // between each send (if 'sleep' is zero, do not sleep)
      for( int i=0; i<numMsgs; i++ ) {
         // Set the user property 'value' to something between 0 and 100
         msg.setIntProperty("value",(int)(rnd.nextFloat()*101));
         publisher.publish(msg);

         if( i%(numMsgs/10)==0 ) System.out.print(".");

         if( sleepTime!=0 ) {
            try {
               Thread.sleep(sleepTime);
            } catch( InterruptedException ie ) {
               // Ignore it
            }
         }
      }

      // Finally close the TopicConnection and quit
      conn.close();
      System.out.println(" Done!");
   }

   /**************************************************************************
    *  parseArguments - Parse the given argument array, updating member      *
    *                   variables as we go. Terminates the application if an *
    *                   error is encountered.                                *
    **************************************************************************/
   private static void parseArguments( String[] args ) {
      try {
         for( int i=0; i<args.length; i++ ) {
            String arg = args[i].toUpperCase();

            if( arg.equals("-NOJNDI"))         useJNDI   = false;
            else if( arg.equals("-URL"))       url       = args[++i];
            else if( arg.equals("-ICF"))       icf       = args[++i];
            else if( arg.equals("-TCFLOOKUP")) tcfLookup = args[++i];
            else if( arg.equals("-TLOOKUP"))   tLookup   = args[++i];
            else if( arg.equals("-T"))         tName     = args[++i];
            else if( arg.equals("-QMGR"))      qMgrName  = args[++i];
            else if( arg.equals("-MSGS"))      numMsgs   = Integer.parseInt(
                                                                   args[++i]);
            else if( arg.equals("-SLEEP"))     sleepTime = Integer.parseInt(
                                                                   args[++i]);
            else
               throw new Exception("Unrecognised argument ("+arg+")");
         }
      } catch( Exception e ) {
         System.out.println("Error parsing arguments: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }

   /**************************************************************************
    *  obtainAdminObjects - Obtains the two administered objects we need and *
    *                       places references to them in the given array.    *
    *                       The method used (JNDI versus explicit construct- *
    *                       ion) is determined by the value of 'useJNDI'.    *
    *                                                                        *
    *                       The application will terminate if any problems   *
    *                       are encountered.                                 *
    **************************************************************************/
   private static void obtainAdminObjects( Object[] outArray ) {
      try {
         if( useJNDI ) {
            // Set up JNDI environment parameters
            Hashtable env = new Hashtable();
            env.put( Context.PROVIDER_URL, url );
            env.put( Context.INITIAL_CONTEXT_FACTORY, icf );

            // Access the JNDI namespace
            Context ctx = new InitialContext(env);

            // Obtain the objects
            outArray[0] = ctx.lookup(tcfLookup);
            outArray[1] = ctx.lookup(tLookup);

         } else {
            // Create the objects explicitly - we need to refer to the
            // MQJMS-specific classes now
            com.ibm.mq.jms.MQTopicConnectionFactory tcf =
                                 new com.ibm.mq.jms.MQTopicConnectionFactory();
            tcf.setQueueManager(qMgrName);
            tcf.setBrokerQueueManager(qMgrName);

            com.ibm.mq.jms.MQTopic t = new com.ibm.mq.jms.MQTopic();
            t.setBaseTopicName(tName);

            outArray[0] = tcf;
            outArray[1] = t;
         }
      } catch( Exception e ) {
         System.out.println("Error obtaining admin objects: "+e+"\nQuitting.");
         System.exit(-1);
      }
   }
}
